var express  = require('express');
var http     = require('http');
var socketio = require('socket.io');
var app      = express();
var port     = Number(process.env.PORT || 5000);

app.get('/', function (req, res) { res.sendfile(__dirname + '/index.html'); });

var loggedinPeople  = new Array();
var server          = http.createServer(app);
var io              = socketio.listen(server);

server.listen(port, function() {
	console.log("Server running at port: " + port);
});

io.on('connection', function (socket) {
	socket.loggedin = false;

	io.sockets.emit('loggedinPeople', loggedinPeople);
  
	socket.on('set nickname', function (data, callback) {

		data = data.trim().replace(/\s+/g, ' ');
		var oldNickname = socket.nickname;

		if ( data.length < 1 ) return resetNickError(socket, 'Takma ad boş olamaz');
		if ( oldNickname === data) return;
		if ( false !== getNickIndex(data) ) return resetNickError(socket, 'Takma ad kullanımda');

		if ( undefined !== oldNickname ) {
			io.sockets.emit('get message', '<i>"'+oldNickname+'" adını "'+data+'" olarak değiştirdi.</i>');
		}
		else {
			io.sockets.emit('get message', '<i>"'+data+'" kullanıcısı oturum açtı.</i>');
			socket.loggedin = true;
		}

		socket.nickname = data;
		removeNickname(oldNickname);
		loggedinPeople.push(socket.nickname);
		io.sockets.emit('loggedinPeople', loggedinPeople);

		socket.emit('update nickname', socket.nickname);
	});

	socket.on('send message', function (message) {
		message = message.trim().replace(/\s+/g, ' ');
		if ( message.length < 1 ) return;
		io.sockets.emit( 'get message', '<b>'+socket.nickname+'</b>: '+message );
	});

	socket.on('disconnect', function () {
		logout (socket, io);
	});

});

function resetNickError (socket, message) {
	if (message && socket.loggedin) {
		socket.emit('update nickname', socket.nickname);
	}
	socket.emit('error message', message);
}

function getNickIndex (nickname) {
	if ( undefined === nickname ) return false;
	var index = loggedinPeople.indexOf(nickname);
	return index === -1 ? false : index;
}

function removeNickname (nickname) {
	var index = getNickIndex(nickname);
	if (false !== index) { loggedinPeople.splice(index, 1) }
}

function logout (socket, io) {
	if (socket.loggedin) {
		removeNickname(socket.nickname);
		io.sockets.emit('get message', '<i>"'+socket.nickname+'" kullanıcısı çıkış yaptı.</i>');
		socket.loggedin = false;
		io.sockets.emit('loggedinPeople', loggedinPeople);
	}
}
